@@ -4,21 +4,20 @@
 from gi.repository import Gtk
 
 import re
-from datetime import date as dateclass
 
+from zim.parse.encode import url_decode, URL_ENCODE_DATA
 from zim.fs import adapt_from_oldfs
 from zim.newfs import LocalFolder
 from zim.plugins import PluginClass
 from zim.actions import action
-from zim.config import data_file, ConfigManager
+from zim.config import ConfigManager
 from zim.notebook import Path, Notebook, NotebookInfo, \
 	resolve_notebook, build_notebook
 from zim.templates import get_template
-from zim.main import GtkCommand, ZIM_APPLICATION
+from zim.main import GtkCommand
 
 from zim.gui.mainwindow import MainWindowExtension
-from zim.gui.widgets import Dialog, ScrolledTextView, IconButton, \
-	InputForm, QuestionDialog
+from zim.gui.widgets import Dialog, ScrolledTextView, InputForm, QuestionDialog
 from zim.gui.clipboard import Clipboard, SelectionClipboard
 from zim.gui.notebookdialog import NotebookComboBox
 
@@ -28,7 +27,23 @@ import logging
 logger = logging.getLogger('zim.plugins.quicknote')
 
 
-usagehelp = '''\
+class QuickNotePluginCommand(GtkCommand):
+
+	options = (
+		('notebook=', '', 'Select the notebook in the dialog'),
+		('page=', '', 'Fill in full page name'),
+		('section=', '', 'Fill in the page section in the dialog'),
+		('namespace=', '', 'Fill in the page section in the dialog'), # backward compatibility
+		('basename=', '', 'Fill in the page name in the dialog'),
+		('append=', '', 'Set whether to append or create new page ("true" or "false")'),
+		('text=', '', 'Provide the text directly'),
+		('input=', '', 'Provide the text on stdin ("stdin") or take the text from the clipboard ("clipboard")'),
+		('encoding=', '', 'Text encoding ("base64" or "url")'),
+		('attachments=', '', 'Import all files in FOLDER as attachments, wiki input can refer these files relatively'),
+		('option=', '', 'Set template parameter, e.g. "url=URL"'),
+	)
+
+	cmdhelp = '''\
 usage: zim --plugin quicknote [OPTIONS]
 
 Options:
@@ -49,41 +64,17 @@ Options:
   --option url=STRING    Set template parameter
 '''
 
-
-class QuickNotePluginCommand(GtkCommand):
-
-	options = (
-		('help', 'h', 'Print this help text and exit'),
-		('notebook=', '', 'Select the notebook in the dialog'),
-		('page=', '', 'Fill in full page name'),
-		('section=', '', 'Fill in the page section in the dialog'),
-		('namespace=', '', 'Fill in the page section in the dialog'), # backward compatibility
-		('basename=', '', 'Fill in the page name in the dialog'),
-		('append=', '', 'Set whether to append or create new page ("true" or "false")'),
-		('text=', '', 'Provide the text directly'),
-		('input=', '', 'Provide the text on stdin ("stdin") or take the text from the clipboard ("clipboard")'),
-		('encoding=', '', 'Text encoding ("base64" or "url")'),
-		('attachments=', '', 'Import all files in FOLDER as attachments, wiki input can refer these files relatively'),
-		('option=', '', 'Set template parameter, e.g. "url=URL"'),
-	)
+	def handle_local_commandline(self, args):
+		text = self.get_text_local()
+		args.extend(['--text', text])
+			# In subsequent parsing, this argument should supersede already in the input
+			# cleanup of the inputs is not trivial, so just leave un-used arguments in
+		return args
 
 	def parse_options(self, *args):
 		self.opts['option'] = [] # allow list
 
-		if all(not a.startswith('-') for a in args):
-			# Backward compartibility for options not prefixed by "--"
-			# used "=" as separator for values
-			# template options came as "option:KEY=VALUE"
-			for arg in args:
-				if arg.startswith('option:'):
-					self.opts['option'].append(arg[7:])
-				elif arg == 'help':
-					self.opts['help'] = True
-				else:
-					key, value = arg.split('=', 1)
-					self.opts[key] = value
-		else:
-			GtkCommand.parse_options(self, *args)
+		GtkCommand.parse_options(self, *args)
 
 		self.template_options = {}
 		for arg in self.opts['option']:
@@ -98,26 +89,26 @@ class QuickNotePluginCommand(GtkCommand):
 			folderpath = LocalFolder(self.pwd).get_abspath(self.opts['attachments'])
 			self.opts['attachments'] = LocalFolder(folderpath)
 
-	def get_text(self):
-		if 'input' in self.opts:
+	def get_text_local(self):
+		if 'text' in self.opts: 
+			# Check this one first, to allow parse_local_commandline() to work
+			text = self.opts['text']
+		elif 'input' in self.opts:
 			if self.opts['input'] == 'stdin':
 				import sys
 				text = sys.stdin.read()
 			elif self.opts['input'] == 'clipboard':
-				text = \
-					SelectionClipboard.get_text() \
-					or Clipboard.get_text()
+				text = SelectionClipboard.get_text() or Clipboard.get_text()
 			else:
 				raise AssertionError('Unknown input type: %s' % self.opts['input'])
 		else:
-			text = self.opts.get('text', '')
+			text = ''
 
 		if text and 'encoding' in self.opts:
 			if self.opts['encoding'] == 'base64':
 				import base64
 				text = base64.b64decode(text).decode('UTF-8')
 			elif self.opts['encoding'] == 'url':
-				from zim.parsing import url_decode, URL_ENCODE_DATA
 				text = url_decode(text, mode=URL_ENCODE_DATA)
 			else:
 				raise AssertionError('Unknown encoding: %s' % self.opts['encoding'])
@@ -125,23 +116,7 @@ class QuickNotePluginCommand(GtkCommand):
 		assert isinstance(text, str), '%r is not decoded' % text
 		return text
 
-	def run_local(self):
-		# Try to run dialog from local process
-		# - prevents issues where dialog pop behind other applications
-		#   (desktop preventing new window of existing process to hijack focus)
-		# - e.g. capturing stdin requires local process
-		if self.opts.get('help'):
-			print(usagehelp) # TODO handle this in the base class
-		else:
-			dialog = self.build_dialog()
-			dialog.run()
-		return True # Done - Don't call run() as well
-
 	def run(self):
-		# If called from primary process just run the dialog
-		return self.build_dialog()
-
-	def build_dialog(self):
 		if 'notebook' in self.opts:
 			notebook = resolve_notebook(self.opts['notebook'])
 		else:
@@ -152,7 +127,7 @@ class QuickNotePluginCommand(GtkCommand):
 			namespace=self.opts.get('namespace'),
 			basename=self.opts.get('basename'),
 			append=self.opts.get('append'),
-			text=self.get_text(),
+			text=self.opts.get('text', ''),
 			template_options=self.template_options,
 			attachments=self.opts.get('attachments')
 		)
@@ -188,7 +163,6 @@ class QuickNoteMainWindowExtension(MainWindowExtension):
 
 
 class QuickNoteDialog(Dialog):
-	'''Dialog bound to a specific notebook'''
 
 	def __init__(self, window, notebook=None,
 		page=None, namespace=None, basename=None,
@@ -308,7 +282,7 @@ class QuickNoteDialog(Dialog):
 		buffer = self.textview.get_buffer()
 		buffer.set_text(''.join(lines))
 		begin, end = buffer.get_bounds()
-		buffer.place_cursor(begin)
+		buffer.place_cursor(end)
 
 		buffer.set_modified(False)
 
@@ -390,13 +364,18 @@ class QuickNoteDialog(Dialog):
 			# Automatically generate a (valid) page name
 			self._updating_title = True
 			start, end = buffer.get_bounds()
-			title = start.get_text(end).strip()[:50]
-				# Cut off at 50 characters to prevent using a whole paragraph
-			title = title.replace(':', '')
-			if '\n' in title:
-				title, _ = title.split('\n', 1)
+			text = start.get_text(end).strip()
+			# Limit page name to contents of first line
+			title = text.split('\n')[0]
+			# Get heading 1 as the page name, if present
+			heading_pattern = r'={1,}\s*(.*?)\s*={1,}'
+			heading_match = re.search(heading_pattern, title)
+			if heading_match:
+				title = heading_match.group(1)
+			# Remove colons and limit to 50 characters
+			title = title.replace(':', '')[:50]
 			try:
-				title = Path.makeValidPageName(title.replace(':', ''))
+				title = Path.makeValidPageName(title)
 				self.form['basename'] = title
 			except ValueError:
 				pass
@@ -439,7 +418,7 @@ class QuickNoteDialog(Dialog):
 
 		if self.open_page_check.get_active():
 			self.hide()
-			ZIM_APPLICATION.present(notebook, path)
+			self.get_application().open_notebook(notebook, path)
 
 		return True
 
@@ -458,8 +437,8 @@ class QuickNoteDialog(Dialog):
 		page.parse('wiki', text, append=True) # FIXME format hard coded
 		notebook.store_page(page)
 
-	def import_attachments(self, notebook, path, dir):
-		dir = adapt_from_oldfs(fir)
+	def import_attachments(self, notebook, path, from_dir):
+		from_dir = adapt_from_oldfs(from_dir)
 		attachments = notebook.get_attachments_dir(path)
-		for name in dir.list_files():
+		for file in from_dir.list_files():
 			file.copyto(attachments)
